/*! 
 * @file  Kreuzung.cpp
 * @brief Implementierung der Klasse Kreuzung.
 */

#include <iostream>
#include <fstream>
using namespace std;

#include "Kreuzung.h"
#include "Konstanten.h"


/*!
 * @fn		Kreuzung::Kreuzung( long int id = 0 )
 * @brief	Default-Konstruktor der Klasse Kreuzung
 *
 * @author	Nico Schroeder
 * @param	id Eine moegliche ID fuer die Kreuzung ... zum wiederfinden in der Statistik
 */
Kreuzung::Kreuzung( long int id ){
	clear();
	idBenutzer_ = id;
}

/*!
 * @fn		void Kreuzung::aufraeumen()
 * @brief	Muss gestartet werden, bevor eine Kreuzung entfernt wird.
 *
 * @author	Nico Schroeder
 */
void Kreuzung::aufraeumen(){
}

/*!
 * @fn		void Kreuzung::takt( double zeitintervall )
 * @brief	Gibt den Takt fuer die Amplschaltung vor und waehlt den definierten Ampelalgorithmus aus.
 *
 * @author	Nico Schroeder
 *
 * @param	zeitintervall gibt die Zeit in Sekunden seit des letzten Taktes an
 * @throw	Fehler* Einen Fehler.
 */
void Kreuzung::takt( double zeitintervall ){
	try{
		switch( idAmpelAlgorithmus_ ){
			case 1: algorithmusZeitZeit( zeitintervall ); break;
			case 2: algorithmusZeitMenge( zeitintervall ); break;
			case 3: algorithmusStopp( zeitintervall ); break;
			case 4: algorithmusMengeMenge( zeitintervall ); break;
		}
	}
	catch( Fehler *fehler ){
		throw( fehler );
	}
}

/*!
 * @fn		void Kreuzung::setzeSpur( int himmelsrichtung, int abbiegerichtung, Spur* spur ){
 * @brief	Fuegt eine Spur an eine bestimmte Stelle in die Kreuzung ein
 *
 * @author	Nico Schroeder
 *
 * @param	himmelsrichtung An welcher Stelle wird die Spur an die Kreuzug gehaengt (OST|SUED|WEST|NORD)
 * @param	abbiegerichtung In welche Richtung geht die Spur (LINKS|MITTE|RECHTS)
 * @param	spur Die einzuhaengende Spur
 *
 * @throw	Fehler* Wirft einen Fehler, wenn die Parameter ausserhalb zulaessiger Grenzen sind
 */
void Kreuzung::setzeSpur( int himmelsrichtung, int abbiegerichtung, Spur* spur ){
	if( himmelsrichtung < 1 || himmelsrichtung > 4 ){
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie Himmelsrichtung fuer eine Spur in einer Kreuzung entspricht nicht den geforderten Parametern.");
		throw(fehler);
	}
	if( abbiegerichtung < 1 || abbiegerichtung > 3 ){
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie Abbiegerichtung fuer eine Spur in einer Kreuzung entspricht nicht den geforderten Parametern.");
		throw(fehler);
	}

	spurenListe_[himmelsrichtung-1][abbiegerichtung-1] = spur;
}

/*!
 * @fn		Spur* Kreuzung::gibSpur( int himmelsrichtung, int abbiegerichtung )
 * @brief	Gibt eine Spur einer bestimmte Stelle in die Kreuzung zurueck
 *
 * @author	Nico Schroeder
 *
 * @param	himmelsrichtung An welcher Stelle wird die Spur an die Kreuzug gehaengt (OST|SUED|WEST|NORD)
 * @param	abbiegerichtung In welche Richtung geht die Spur (LINKS|MITTE|RECHTS)
 *
 * @throw	fehler Wirft einen Fehler, wenn die Parameter ausserhalb zulaessiger Grenzen sind
 *
 * @return	Spur Gibt die entsprechende Spur zurueck
 */
Spur* Kreuzung::gibSpur( int himmelsrichtung, int abbiegerichtung ){
	if( himmelsrichtung < 1 || himmelsrichtung > 4 ){
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie Himmelsrichtung fuer eine Spur in einer Kreuzung entspricht nicht den geforderten Parametern.");
		throw(fehler);
	}
	if( abbiegerichtung < 1 || abbiegerichtung > 3 ){
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie Abbiegerichtung fuer eine Spur in einer Kreuzung entspricht nicht den geforderten Parametern.");
		throw(fehler);
	}

	return spurenListe_[himmelsrichtung-1][abbiegerichtung-1];
}


/*
 * @fn		void Kreuzung::inString( )
 * @brief	Gibt die Kreuzung auf der Konsole aus
 *
 * @author	Nico Schroeder
 */
void Kreuzung::inString( ){
	cout << this;
}

/*
 * @fn		void Kreuzung::setzeAmpelalgorithmus( int id )
 * @brief	Gibt die Kreuzung auf der Konsole aus
 *
 * @author	Nico Schroeder
 *
 * @param	id Die ID des Ampelalgorithmus
 * @throw	Fehler* Einen Fehler.
 */
void Kreuzung::setzeAmpelalgorithmus( int id ){
	if( id > 0 && id < 5 ){
		idAmpelAlgorithmus_ = id;
		// Wenn ZeitMenge ausgewaehlt wird, so setzte die Schaltmenge auf eine bestimmte Anzahl Fahrzeuge
		if( idAmpelAlgorithmus_ == 2 )
			schaltmenge_[0] = 5;
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nFuer eine Kreuzung soll ein nicht definierter Ampelalgorithmus verwendet werden.");
		throw(fehler);
	}
}

/*
 * @fn		void Kreuzung::clear()
 * @brief	Setzt alle Attribute auf Defaultwerte zurueck.
 *
 * @author	Nico Schroeder
 */
void Kreuzung::clear(){
int i(0), j(0), k(0);
	for( i = 0; i < 4; i++ ){
		for( j = 0; j < 3; j++ ){
			spurenListe_[i][j] = 0;
		}
	}
	
	for( i = 0; i < 4; i++ ){
		for( j = 0; j < 3; j++ ){
			for( k = 0; k < 4; k++ ){
				statistik_[i][j][0] = 0;
				statistik_[i][j][1] = 0;
				statistik_[i][j][2] = 0;
				statistik_[i][j][3] = 0;
			}
		}
	}
	idAmpelAlgorithmus_ = 1;	// Als default immer den Zeit-Zeit-Algorithmus waehlen
	schaltungstakt_ = 1;
	schaltzeit_ = 30;
	for( i = 0; i < 8; i++ ){
		schaltmenge_[i] = 0;
		schaltreihenfolge_[i] = 0;
	}
	schaltreihenfolge_[0] = 1;
	
	zeitLetzteSchaltungGlobal_ = 9999;
	mengeLetzteSchaltungGlobal_ = 9999;
	loescheMich_ = 0;
	idBenutzer_ = 0;
}

/*!
 * @fn		void Kreuzung::exportieren( string cvsDateiname )
 * @brief	Exportiert die aktuellen Kreuzungsstatistiken in ein CSV-File. Eine schon bestehende Datei wir mit den Daten ergaenzt.
 *
 * @author	Nico Schroeder
 *
 * @param	cvsDateiname Der Pfad und Name zu der abzuspeichernden CSV-Datei
 * @throw	Fehler* Einen Fehler.
 */
void Kreuzung::exportieren( string cvsDateiname ){
	string dateiname = cvsDateiname;
	ofstream cvsDatei;
	int pkw(0), lkw(0), bus(0), bike(0), insgesamt(0);

	int himmelsrichtung(0), richtung(0);
	// Datei oeffnen
	cvsDatei.open( dateiname.c_str(), ios::app );
	if( !cvsDatei ){
		Fehler *fehler = new Fehler( "Die CVS-Datei fuer Kreuzungen konnte nicht geoeffnet werden.\n");
		fehler->ergaenzeMeldung( dateiname );
		throw(fehler);
		return;
	}

	// Daten ausstreamen
	// Kreuzungs-Id
	cvsDatei << "\"\";\"\";\"\";\"\";\"\";\"\";\"\"" << endl;
	cvsDatei << "\"Kreuzungs-Id:\";\"" << idBenutzer_ <<  "\";\"\";\"\";\"\";\"\"" << endl;
	cvsDatei << "\"Algorithmus-Id:\";\"" << idAmpelAlgorithmus_ <<  "\";\"\";\"\";\"\";\"\"" << endl;
	cvsDatei << "\"Spur-Id\";\"BIKE\";\"PKW\";\"LKW\";\"BUS\";\"INSGESAMT\"" << endl;
	// Die einzelnen Spuren rausschreiben
	for( himmelsrichtung = 0; himmelsrichtung < 4; himmelsrichtung++ ){
		for( richtung = 0; richtung < 3; richtung++ ){
			if( spurenListe_[himmelsrichtung][richtung] != 0 ){
				bike = statistik_[himmelsrichtung][richtung][BIKE-1];
				pkw = statistik_[himmelsrichtung][richtung][PKW-1];
				lkw = statistik_[himmelsrichtung][richtung][LKW-1];
				bus = statistik_[himmelsrichtung][richtung][BUS-1];
				insgesamt = bike + pkw + lkw + bus;
				cvsDatei << "\"" << spurenListe_[himmelsrichtung][richtung]->gibId( ) << "\";\"" << bike << "\";\"" << pkw << "\";\"" << lkw << "\";\"" << bus << "\";\"" << insgesamt << "\"" << endl;
			}
		}
	}
	
	// Datei schliessen
	cvsDatei.close();
}

/*!
 * @fn		ostream &operator<< ( ostream &os, Kreuzung *kreuzung )
 * @brief	Ueberladener Operator "<<" z.B. zur Ausgabe einer Kreuzung ueber cout
 *
 * @author	Nico Schroeder
 *
 * @param	os Der zu modifizierende OutStream.
 * @param	kreuzung Die auszugebende Kreuzung.
 * @return	Der modifizierte OutStream.
 */
ostream &operator<< ( ostream &os, Kreuzung *kreuzung ){
	os << "----------Kreuzung----------" << endl;
	os << "ID:                   " << kreuzung->id_ << endl;
	return os;
}

///////////////// AMPELALGORITHMEN /////////////////////////////////////

/*!
 * @fn		void Kreuzung::algorithmusZeitZeit( double zeitintervall )
 * @brief	Fuehrt eine Ampelschaltung nach dem Prinzip Zeit-Zeit durch
 *			Der Algorithmus ist in 8 Takte gegliedert, die nach einander ablaufen: <br>
 *			1. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot<br>
 *			2. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)<br>
 *			3. Ost links, West links, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot<br>
 *			4. Ost links, West links, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)<br>
 *			5. Ost gerade, West gerade, Ost rechts, West rechts bekommen gruen ... alle anderen rot<br>
 *			6. Ost gerade, West gerade, Ost rechts, West rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)<br>
 *			7. Nord links, Sued links, West rechts, Ost rechts bekommen gruen ... alle anderen rot<br>
 *			8. Nord links, Sued links, West rechts, Ost rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
 *
 * @author	Nico Schroeder
 *
 * @param	zeitintervall gibt die Zeit in Sekunden seit des letzten Taktes an
 */
void Kreuzung::algorithmusZeitZeit( double zeitintervall ){
	// Teste, ob die Zeit schon abgelaufen ist - nur globaler Test
	// Bei diesem Algorithmus ist es unwichtig, wann auf welcher Spur das letzte mal geschalten wurde
	bool schalten = 0;
	zeitLetzteSchaltungGlobal_ += zeitintervall;

	switch( schaltungstakt_ ){
		// Ablaufzeit = schaltZeit_
		case 1:
		case 3:
		case 5:
		case 7: if( zeitLetzteSchaltungGlobal_ >= schaltzeit_ )
					schalten = 1;
			    break;
		// Ablaufzeit = 3 Sekunden
		case 2:
		case 4:
		case 6:
		case 8: if( zeitLetzteSchaltungGlobal_ >= 3 )
					schalten = 1;
			    break;
	}

	// Schalte oder schalte nicht
	if( schalten != 0 ){
		schaltungstakt_++;
		if( schaltungstakt_ > 8 )
			schaltungstakt_ = 1;
		zeitLetzteSchaltungGlobal_ = 0;
	
		// Stelle die Ampeln entsprechend des aktuellen Taktes ein
		switch( schaltungstakt_ ){
			//	1. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot
			case 1:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeGruen();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeGruen();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	2. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 2:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeGelb();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeGelb();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	3. Ost links, West links, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot
			case 3:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeGruen();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeGruen();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	4. Ost links, West links, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 4:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeGelb();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeGelb();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	5. Ost gerade, West gerade, Ost rechts, West rechts bekommen gruen ... alle anderen rot
			case 5:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeGruen();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeGruen();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGruen();

					break;
				}
			//	6. Ost gerade, West gerade, Ost rechts, West rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 6:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeGelb();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeGelb();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGelb();

					break;
				}
			//	7. Nord links, Sued links, West rechts, Ost rechts bekommen gruen ... alle anderen rot
			case 7:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeGruen();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeGruen();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGruen();

					break;
				}
			//	8. Nord links, Sued links, West rechts, Ost rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 8:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeGelb();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeGelb();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGelb();

					break;
				}
		}
	}

	// erzeuge die Statistiken
	if( spurenListe_[NORD-1][LINKS-1] != 0 ){
		statistik_[NORD-1][LINKS-1][BIKE-1] += spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[NORD-1][LINKS-1][PKW-1] += spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[NORD-1][LINKS-1][LKW-1] += spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[NORD-1][LINKS-1][BUS-1] += spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}
	if( spurenListe_[NORD-1][MITTE-1] != 0 ){
		statistik_[NORD-1][MITTE-1][BIKE-1] += spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[NORD-1][MITTE-1][PKW-1] += spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[NORD-1][MITTE-1][LKW-1] += spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[NORD-1][MITTE-1][BUS-1] += spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}
	if( spurenListe_[NORD-1][RECHTS-1] != 0 ){
		statistik_[NORD-1][RECHTS-1][BIKE-1] += spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[NORD-1][RECHTS-1][PKW-1] += spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[NORD-1][RECHTS-1][LKW-1] += spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[NORD-1][RECHTS-1][BUS-1] += spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}

	if( spurenListe_[SUED-1][LINKS-1] != 0 ){
		statistik_[SUED-1][LINKS-1][BIKE-1] += spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[SUED-1][LINKS-1][PKW-1] += spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[SUED-1][LINKS-1][LKW-1] += spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[SUED-1][LINKS-1][BUS-1] += spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}
	if( spurenListe_[SUED-1][MITTE-1] != 0 ){
		statistik_[SUED-1][MITTE-1][BIKE-1] += spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[SUED-1][MITTE-1][PKW-1] += spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[SUED-1][MITTE-1][LKW-1] += spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[SUED-1][MITTE-1][BUS-1] += spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}
	if( spurenListe_[SUED-1][RECHTS-1] != 0 ){
		statistik_[SUED-1][RECHTS-1][BIKE-1] += spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[SUED-1][RECHTS-1][PKW-1] += spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[SUED-1][RECHTS-1][LKW-1] += spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[SUED-1][RECHTS-1][BUS-1] += spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}

	if( spurenListe_[OST-1][LINKS-1] != 0 ){
		statistik_[OST-1][LINKS-1][BIKE-1] += spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[OST-1][LINKS-1][PKW-1] += spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[OST-1][LINKS-1][LKW-1] += spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[OST-1][LINKS-1][BUS-1] += spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}
	if( spurenListe_[OST-1][MITTE-1] != 0 ){
		statistik_[OST-1][MITTE-1][BIKE-1] += spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[OST-1][MITTE-1][PKW-1] += spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[OST-1][MITTE-1][LKW-1] += spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[OST-1][MITTE-1][BUS-1] += spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}
	if( spurenListe_[OST-1][RECHTS-1] != 0 ){
		statistik_[OST-1][RECHTS-1][BIKE-1] += spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[OST-1][RECHTS-1][PKW-1] += spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[OST-1][RECHTS-1][LKW-1] += spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[OST-1][RECHTS-1][BUS-1] += spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}

	if( spurenListe_[WEST-1][LINKS-1] != 0 ){
		statistik_[WEST-1][LINKS-1][BIKE-1] += spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[WEST-1][LINKS-1][PKW-1] += spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[WEST-1][LINKS-1][LKW-1] += spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[WEST-1][LINKS-1][BUS-1] += spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}
	if( spurenListe_[WEST-1][MITTE-1] != 0 ){
		statistik_[WEST-1][MITTE-1][BIKE-1] += spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[WEST-1][MITTE-1][PKW-1] += spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[WEST-1][MITTE-1][LKW-1] += spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[WEST-1][MITTE-1][BUS-1] += spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}
	if( spurenListe_[WEST-1][RECHTS-1] != 0 ){
		statistik_[WEST-1][RECHTS-1][BIKE-1] += spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		statistik_[WEST-1][RECHTS-1][PKW-1] += spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		statistik_[WEST-1][RECHTS-1][LKW-1] += spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		statistik_[WEST-1][RECHTS-1][BUS-1] += spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );
	}

}


/*!
 * @fn		void Kreuzung::algorithmusZeitMenge( double zeitintervall )
 * @brief	Fuehrt eine Ampelschaltung nach dem Prinzip Zeit-Menge durch
 *			Der Algorithmus ist in 8 Takte gegliedert, die nach einander ablaufen: <br>
 *			1. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot<br>
 *			2. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)<br>
 *			3. Ost links, West links, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot<br>
 *			4. Ost links, West links, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)<br>
 *			5. Ost gerade, West gerade, Ost rechts, West rechts bekommen gruen ... alle anderen rot<br>
 *			6. Ost gerade, West gerade, Ost rechts, West rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)<br>
 *			7. Nord links, Sued links, West rechts, Ost rechts bekommen gruen ... alle anderen rot<br>
 *			8. Nord links, Sued links, West rechts, Ost rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)<br>
 *
 * @author	Nico Schroeder
 *
 * @param	zeitintervall gibt die Zeit in Sekunden seit des letzten Taktes an
 */
void Kreuzung::algorithmusZeitMenge( double zeitintervall ){
	// Teste, ob die Zeit schon abgelaufen ist - nur globaler Test
	// Bei diesem Algorithmus ist es unwichtig, wann auf welcher Spur das letzte mal geschalten wurde
	bool schalten = 0;
	zeitLetzteSchaltungGlobal_ += zeitintervall;

	// Entscheidung, ob wieder geschaltet werden muss
	switch( schaltungstakt_ ){
		// entweder ist die Zeit abgelaufen, oder eine bestimmte Anzahl Fahrzeuge ueber die Ampel gefahren
		case 1:
		case 3:
		case 5:
		case 7: if( zeitLetzteSchaltungGlobal_ >= schaltzeit_ || mengeLetzteSchaltungGlobal_ >= schaltmenge_[0] ){
					schalten = 1;
					zeitLetzteSchaltungGlobal_ = 0;
					mengeLetzteSchaltungGlobal_ = 0;
				}
				break;
		// Ablaufzeit = 3 Sekunden
		case 2:
		case 4:
		case 6:
		case 8: if( zeitLetzteSchaltungGlobal_ >= 3 ){
					schalten = 1;
					zeitLetzteSchaltungGlobal_ = 0;
					mengeLetzteSchaltungGlobal_ = 0;
				}
			    break;
	}


	// Schalte oder schalte nicht
	if( schalten != 0 ){
		schaltungstakt_++;
		if( schaltungstakt_ > 8 )
			schaltungstakt_ = 1;
		zeitLetzteSchaltungGlobal_ = 0;
		mengeLetzteSchaltungGlobal_ = 0;
	
		// Stelle die Ampeln entsprechend des aktuellen Taktes ein
		switch( schaltungstakt_ ){
			//	1. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot
			case 1:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeGruen();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeGruen();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	2. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 2:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeGelb();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeGelb();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	3. Ost links, West links, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot
			case 3:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeGruen();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeGruen();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	4. Ost links, West links, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 4:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeGelb();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeGelb();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	5. Ost gerade, West gerade, Ost rechts, West rechts bekommen gruen ... alle anderen rot
			case 5:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeGruen();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeGruen();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGruen();

					break;
				}
			//	6. Ost gerade, West gerade, Ost rechts, West rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 6:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeGelb();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeGelb();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGelb();

					break;
				}
			//	7. Nord links, Sued links, West rechts, Ost rechts bekommen gruen ... alle anderen rot
			case 7:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeGruen();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeGruen();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGruen();

					break;
				}
			//	8. Nord links, Sued links, West rechts, Ost rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 8:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeGelb();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeGelb();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGelb();

					break;
				}
		}
	}
	int anzahlBike(0), anzahlPkw(0), anzahlLkw(0), anzahlBus(0);
	// erzeuge die Statistiken
	if( spurenListe_[NORD-1][LINKS-1] != 0 ){
		anzahlBike = spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[NORD-1][LINKS-1][BIKE-1] += anzahlBike;
		statistik_[NORD-1][LINKS-1][PKW-1] += anzahlPkw;
		statistik_[NORD-1][LINKS-1][LKW-1] += anzahlLkw;
		statistik_[NORD-1][LINKS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[NORD-1][MITTE-1] != 0 ){
		anzahlBike = spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[NORD-1][MITTE-1][BIKE-1] += anzahlBike;
		statistik_[NORD-1][MITTE-1][PKW-1] += anzahlPkw;
		statistik_[NORD-1][MITTE-1][LKW-1] += anzahlLkw;
		statistik_[NORD-1][MITTE-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[NORD-1][RECHTS-1] != 0 ){
		anzahlBike = spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[NORD-1][RECHTS-1][BIKE-1] += anzahlBike;
		statistik_[NORD-1][RECHTS-1][PKW-1] += anzahlPkw;
		statistik_[NORD-1][RECHTS-1][LKW-1] += anzahlLkw;
		statistik_[NORD-1][RECHTS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}

	if( spurenListe_[SUED-1][LINKS-1] != 0 ){
		anzahlBike = spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[SUED-1][LINKS-1][BIKE-1] += anzahlBike;
		statistik_[SUED-1][LINKS-1][PKW-1] += anzahlPkw;
		statistik_[SUED-1][LINKS-1][LKW-1] += anzahlLkw;
		statistik_[SUED-1][LINKS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[SUED-1][MITTE-1] != 0 ){
		anzahlBike = spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[SUED-1][MITTE-1][BIKE-1] += anzahlBike;
		statistik_[SUED-1][MITTE-1][PKW-1] += anzahlPkw;
		statistik_[SUED-1][MITTE-1][LKW-1] += anzahlLkw;
		statistik_[SUED-1][MITTE-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[SUED-1][RECHTS-1] != 0 ){
		anzahlBike = spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[SUED-1][RECHTS-1][BIKE-1] += anzahlBike;
		statistik_[SUED-1][RECHTS-1][PKW-1] += anzahlPkw;
		statistik_[SUED-1][RECHTS-1][LKW-1] += anzahlLkw;
		statistik_[SUED-1][RECHTS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}

	if( spurenListe_[OST-1][LINKS-1] != 0 ){
		anzahlBike = spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[OST-1][LINKS-1][BIKE-1] += anzahlBike;
		statistik_[OST-1][LINKS-1][PKW-1] += anzahlPkw;
		statistik_[OST-1][LINKS-1][LKW-1] += anzahlLkw;
		statistik_[OST-1][LINKS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[OST-1][MITTE-1] != 0 ){
		anzahlBike = spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[OST-1][MITTE-1][BIKE-1] += anzahlBike;
		statistik_[OST-1][MITTE-1][PKW-1] += anzahlPkw;
		statistik_[OST-1][MITTE-1][LKW-1] += anzahlLkw;
		statistik_[OST-1][MITTE-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[OST-1][RECHTS-1] != 0 ){
		anzahlBike = spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );


		statistik_[OST-1][RECHTS-1][BIKE-1] += anzahlBike;
		statistik_[OST-1][RECHTS-1][PKW-1] += anzahlPkw;
		statistik_[OST-1][RECHTS-1][LKW-1] += anzahlLkw;
		statistik_[OST-1][RECHTS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}

	if( spurenListe_[WEST-1][LINKS-1] != 0 ){
		anzahlBike = spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[WEST-1][LINKS-1][BIKE-1] += anzahlBike;
		statistik_[WEST-1][LINKS-1][PKW-1] += anzahlPkw;
		statistik_[WEST-1][LINKS-1][LKW-1] += anzahlLkw;
		statistik_[WEST-1][LINKS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[WEST-1][MITTE-1] != 0 ){
		anzahlBike = spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[WEST-1][MITTE-1][BIKE-1] += anzahlBike;
		statistik_[WEST-1][MITTE-1][PKW-1] += anzahlPkw;
		statistik_[WEST-1][MITTE-1][LKW-1] += anzahlLkw;
		statistik_[WEST-1][MITTE-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[WEST-1][RECHTS-1] != 0 ){
		anzahlBike = spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[WEST-1][RECHTS-1][BIKE-1] += anzahlBike;
		statistik_[WEST-1][RECHTS-1][PKW-1] += anzahlPkw;
		statistik_[WEST-1][RECHTS-1][LKW-1] += anzahlLkw;
		statistik_[WEST-1][RECHTS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}

}
/*!
 * @fn		void Kreuzung::algorithmusMengeMenge( double zeitintervall )
 * @brief	Dieser Algorithmus gleicht einer Klospuelung. Es wird zu einem bestimmten Zeitpunkt geschaut, wieviele
 *			Fahrzeuge auf der Kreuzung warten. Die Schaltung wird dann priorisiert nach der Menge vor den jeweiligen
 *			Ampeln so geschaltet, dass alle Fahrzeuge, die zum Zeitpunkt t dort stehen rueber kommen.
 *
 * @author	Nico Schroeder
 *
 * @param	zeitintervall gibt die Zeit in Sekunden seit des letzten Taktes an
 */
void Kreuzung::algorithmusMengeMenge( double zeitintervall ){
	bool schalten( 0 );
	int anzahlFahrzeuge[4], i(0);
	zeitLetzteSchaltungGlobal_ += zeitintervall;
	
	switch( schaltreihenfolge_[schaltungstakt_-1] ){
		case 1:
		case 3:
		case 5:
		case 7:{
					// Teste, ob aus der aktuellen Spur alle Fahrzeuge herausgefahren sind
					if( mengeLetzteSchaltungGlobal_ < schaltmenge_[schaltungstakt_-1] ){
						// Wenn nein ... tue nichts
						schalten = 0;
					}
					else{
						// Wenn ja, ueberpruefe ob noch 3 Sekunden vergangen sind und schalte dann ==> gehe im Takt eins weiter
						if( zeitLetzteSchaltungGlobal_ > 3 )
							schalten = 1;
					}
					break;
			   }
		case 2:
		case 4:
		case 6:
		case 8:{
					if( zeitLetzteSchaltungGlobal_ < 3 ){
						// Wenn nein ... tue nichts
						schalten = 0;
					}
					else{
						// Wenn ja, schalte ==> gehe im Takt eins weiter
						schalten = 1;
					}
					break;
			   }
	}
	

	if( schalten == 1 ){
		
		mengeLetzteSchaltungGlobal_ = 0;
		zeitLetzteSchaltungGlobal_ = 0;

		// In der Schaltung einen Schritt weiter gehen
		if( schaltmenge_[schaltungstakt_-1] != 0 ){
			schaltungstakt_++;
			// die Zaehlvariable und den letzten Takt zurueck setzten
			if( schaltungstakt_ > 1 )
				schaltmenge_[schaltungstakt_-2] = 0;
		}
		
		// Sind alle Takte abgelaufen, so stelle eine neue Betrachtung an
		if( schaltungstakt_ > 8 || schaltmenge_[schaltungstakt_-1] == 0 ){
			anzahlFahrzeuge[0] = 0;
			anzahlFahrzeuge[1] = 0;
			anzahlFahrzeuge[2] = 0;
			anzahlFahrzeuge[3] = 0;
			// Betrachtung der Spuren des 1. Taktes aus ZeitZeit
			if( spurenListe_[NORD-1][MITTE-1] != 0 )
				anzahlFahrzeuge[0] += spurenListe_[NORD-1][MITTE-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[NORD-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[0] += spurenListe_[NORD-1][RECHTS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[SUED-1][MITTE-1] != 0 )
				anzahlFahrzeuge[0] += spurenListe_[SUED-1][MITTE-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[SUED-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[0] += spurenListe_[SUED-1][RECHTS-1]->gibAnzahlFahrzeuge();

			// Betrachtung der Spuren des 3. Taktes aus ZeitZeit
			if( spurenListe_[NORD-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[1] += spurenListe_[NORD-1][RECHTS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[SUED-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[1] += spurenListe_[SUED-1][RECHTS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[OST-1][LINKS-1] != 0 )
				anzahlFahrzeuge[1] += spurenListe_[OST-1][LINKS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[WEST-1][LINKS-1] != 0 )
				anzahlFahrzeuge[1] += spurenListe_[WEST-1][LINKS-1]->gibAnzahlFahrzeuge();

			// Betrachtung der Spuren des 5. Taktes aus ZeitZeit
			if( spurenListe_[OST-1][MITTE-1] != 0 )
				anzahlFahrzeuge[2] += spurenListe_[OST-1][MITTE-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[OST-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[2] += spurenListe_[OST-1][RECHTS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[WEST-1][MITTE-1] != 0 )
				anzahlFahrzeuge[2] += spurenListe_[WEST-1][MITTE-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[WEST-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[2] += spurenListe_[WEST-1][RECHTS-1]->gibAnzahlFahrzeuge();

			// Betrachtung der Spuren des 7. Taktes aus ZeitZeit
			if( spurenListe_[NORD-1][LINKS-1] != 0 )
				anzahlFahrzeuge[3] += spurenListe_[NORD-1][LINKS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[SUED-1][LINKS-1] != 0 )
				anzahlFahrzeuge[3] += spurenListe_[SUED-1][LINKS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[OST-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[3] += spurenListe_[OST-1][RECHTS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[WEST-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[3] += spurenListe_[WEST-1][RECHTS-1]->gibAnzahlFahrzeuge();

			// Da die rechten Spuren zweimal eingerechnet wurden, muessen diese in einem der Takte abgezogen werden
			// Dies wird in dem Takt gemacht, in dem weniger Fahrzeuge die Kreuzung passieren koennen
			if( anzahlFahrzeuge[0] <= anzahlFahrzeuge[1] )
				i = 0;
			else
				i = 1;

			if( spurenListe_[NORD-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[i] -= spurenListe_[NORD-1][RECHTS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[SUED-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[i] -= spurenListe_[SUED-1][RECHTS-1]->gibAnzahlFahrzeuge();

			if( anzahlFahrzeuge[2] <= anzahlFahrzeuge[3] )
				i = 2;
			else
				i = 3;

			if( spurenListe_[WEST-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[i] -= spurenListe_[WEST-1][RECHTS-1]->gibAnzahlFahrzeuge();
			if( spurenListe_[OST-1][RECHTS-1] != 0 )
				anzahlFahrzeuge[i] -= spurenListe_[OST-1][RECHTS-1]->gibAnzahlFahrzeuge();


			// Vergleiche die moegliche Anzahl in jedem Takt. Falls ein Takt keine Fahrzeuge befoerdern wuerde, so wird er ausgelassen
			// Der Takt, welcher die meisten Fahrzeuge befoerdern kann, ist der erste auszufuehrende und so weiter
			schaltungstakt_ = 0;
			for( i = 0; i<4; i++ ){
				int zuSetzenderTakt( 0 );
				// Waehle den aktuell zu setztenden Takt aus
				if( anzahlFahrzeuge[0] > anzahlFahrzeuge[1] )
					zuSetzenderTakt = 0;
				else
					zuSetzenderTakt = 1;

				if( anzahlFahrzeuge[2] > anzahlFahrzeuge[zuSetzenderTakt] )
					zuSetzenderTakt = 2;
				if( anzahlFahrzeuge[3] > anzahlFahrzeuge[zuSetzenderTakt] )
					zuSetzenderTakt = 3;

				// setze den "gruen"-Takt
				schaltreihenfolge_[schaltungstakt_] = zuSetzenderTakt * 2 + 1;
				schaltmenge_[schaltungstakt_] = anzahlFahrzeuge[zuSetzenderTakt];
				schaltungstakt_++;
				
				// setze den "gelb"-Takt
				schaltreihenfolge_[schaltungstakt_] = zuSetzenderTakt * 2 + 2;
				schaltmenge_[schaltungstakt_] = anzahlFahrzeuge[zuSetzenderTakt];
				schaltungstakt_++;

				// setze die Anzahl der Fahrzeuge im verwendeten Takt auf 0, damit beim naechsten Durchlauf ein anderer Takt gefunden wird
				anzahlFahrzeuge[zuSetzenderTakt] = 0;
			}

			// Stelle den Takt auf die erste Stelle
			schaltungstakt_ = 1;
		}

		// Stelle die Ampeln entsprechend des aktuellen Taktes ein
		switch( schaltreihenfolge_[schaltungstakt_-1] ){
			//	1. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot
			case 1:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeGruen();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeGruen();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	2. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 2:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeGelb();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeGelb();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	3. Ost links, West links, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot
			case 3:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeGruen();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeGruen();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	4. Ost links, West links, Nord rechts, Sued rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 4:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeGelb();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeGelb();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}
			//	5. Ost gerade, West gerade, Ost rechts, West rechts bekommen gruen ... alle anderen rot
			case 5:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeGruen();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeGruen();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGruen();

					break;
				}
			//	6. Ost gerade, West gerade, Ost rechts, West rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 6:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeGelb();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeGelb();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGelb();

					break;
				}
			//	7. Nord links, Sued links, West rechts, Ost rechts bekommen gruen ... alle anderen rot
			case 7:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeGruen();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeGruen();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGruen();

					break;
				}
			//	8. Nord links, Sued links, West rechts, Ost rechts bekommen gelb ... alle anderen rot (fuer 3 Sekunden)
			case 8:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeGelb();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeGelb();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGelb();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGelb();

					break;
				}
		}
	}

	int anzahlBike(0), anzahlPkw(0), anzahlLkw(0), anzahlBus(0);
	// erzeuge die Statistiken
	if( spurenListe_[NORD-1][LINKS-1] != 0 ){
		anzahlBike = spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[NORD-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[NORD-1][LINKS-1][BIKE-1] += anzahlBike;
		statistik_[NORD-1][LINKS-1][PKW-1] += anzahlPkw;
		statistik_[NORD-1][LINKS-1][LKW-1] += anzahlLkw;
		statistik_[NORD-1][LINKS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[NORD-1][MITTE-1] != 0 ){
		anzahlBike = spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[NORD-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[NORD-1][MITTE-1][BIKE-1] += anzahlBike;
		statistik_[NORD-1][MITTE-1][PKW-1] += anzahlPkw;
		statistik_[NORD-1][MITTE-1][LKW-1] += anzahlLkw;
		statistik_[NORD-1][MITTE-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[NORD-1][RECHTS-1] != 0 ){
		anzahlBike = spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[NORD-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[NORD-1][RECHTS-1][BIKE-1] += anzahlBike;
		statistik_[NORD-1][RECHTS-1][PKW-1] += anzahlPkw;
		statistik_[NORD-1][RECHTS-1][LKW-1] += anzahlLkw;
		statistik_[NORD-1][RECHTS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}

	if( spurenListe_[SUED-1][LINKS-1] != 0 ){
		anzahlBike = spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[SUED-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[SUED-1][LINKS-1][BIKE-1] += anzahlBike;
		statistik_[SUED-1][LINKS-1][PKW-1] += anzahlPkw;
		statistik_[SUED-1][LINKS-1][LKW-1] += anzahlLkw;
		statistik_[SUED-1][LINKS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[SUED-1][MITTE-1] != 0 ){
		anzahlBike = spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[SUED-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[SUED-1][MITTE-1][BIKE-1] += anzahlBike;
		statistik_[SUED-1][MITTE-1][PKW-1] += anzahlPkw;
		statistik_[SUED-1][MITTE-1][LKW-1] += anzahlLkw;
		statistik_[SUED-1][MITTE-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[SUED-1][RECHTS-1] != 0 ){
		anzahlBike = spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[SUED-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[SUED-1][RECHTS-1][BIKE-1] += anzahlBike;
		statistik_[SUED-1][RECHTS-1][PKW-1] += anzahlPkw;
		statistik_[SUED-1][RECHTS-1][LKW-1] += anzahlLkw;
		statistik_[SUED-1][RECHTS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}

	if( spurenListe_[OST-1][LINKS-1] != 0 ){
		anzahlBike = spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[OST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[OST-1][LINKS-1][BIKE-1] += anzahlBike;
		statistik_[OST-1][LINKS-1][PKW-1] += anzahlPkw;

		statistik_[OST-1][LINKS-1][LKW-1] += anzahlLkw;
		statistik_[OST-1][LINKS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[OST-1][MITTE-1] != 0 ){
		anzahlBike = spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[OST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[OST-1][MITTE-1][BIKE-1] += anzahlBike;
		statistik_[OST-1][MITTE-1][PKW-1] += anzahlPkw;
		statistik_[OST-1][MITTE-1][LKW-1] += anzahlLkw;
		statistik_[OST-1][MITTE-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[OST-1][RECHTS-1] != 0 ){
		anzahlBike = spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[OST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[OST-1][RECHTS-1][BIKE-1] += anzahlBike;
		statistik_[OST-1][RECHTS-1][PKW-1] += anzahlPkw;
		statistik_[OST-1][RECHTS-1][LKW-1] += anzahlLkw;
		statistik_[OST-1][RECHTS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}

	if( spurenListe_[WEST-1][LINKS-1] != 0 ){
		anzahlBike = spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[WEST-1][LINKS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[WEST-1][LINKS-1][BIKE-1] += anzahlBike;
		statistik_[WEST-1][LINKS-1][PKW-1] += anzahlPkw;
		statistik_[WEST-1][LINKS-1][LKW-1] += anzahlLkw;
		statistik_[WEST-1][LINKS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[WEST-1][MITTE-1] != 0 ){
		anzahlBike = spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[WEST-1][MITTE-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[WEST-1][MITTE-1][BIKE-1] += anzahlBike;
		statistik_[WEST-1][MITTE-1][PKW-1] += anzahlPkw;
		statistik_[WEST-1][MITTE-1][LKW-1] += anzahlLkw;
		statistik_[WEST-1][MITTE-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
	if( spurenListe_[WEST-1][RECHTS-1] != 0 ){
		anzahlBike = spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BIKE );
		anzahlPkw = spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( PKW );
		anzahlLkw = spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( LKW );
		anzahlBus = spurenListe_[WEST-1][RECHTS-1]->gibAnzahlAbgefahrenerFahrzeuge( BUS );

		statistik_[WEST-1][RECHTS-1][BIKE-1] += anzahlBike;
		statistik_[WEST-1][RECHTS-1][PKW-1] += anzahlPkw;
		statistik_[WEST-1][RECHTS-1][LKW-1] += anzahlLkw;
		statistik_[WEST-1][RECHTS-1][BUS-1] += anzahlBus;
		
		mengeLetzteSchaltungGlobal_ = anzahlBike + anzahlPkw + anzahlLkw + anzahlBus + mengeLetzteSchaltungGlobal_;
	}
}

/*!
 * @fn		void Kreuzung::algorithmusStopp( double zeitintervall )
 * @brief	Dieser Algortihmus dient nur zu Testzwecken
 *			Der Algorithmus ist in 2 Takte gegliedert, die nach einander ablaufen: <br>
 *			1. alle bekommen gruen ... niemand rot<br>
 *			2. alle bekommen rot ... niemand gruen
 *
 * @author	Nico Schroeder
 *
 * @param	zeitintervall gibt die Zeit in Sekunden seit des letzten Taktes an
 */
void Kreuzung::algorithmusStopp( double zeitintervall ){
	// Teste, ob die Zeit schon abgelaufen ist - nur globaler Test
	// Bei diesem Algorithmus ist es unwichtig, wann auf welcher Spur das letzte mal geschalten wurde
	bool schalten = 0;
	zeitLetzteSchaltungGlobal_ += zeitintervall;

	// Entscheidung, ob wieder geschaltet werden muss
	switch( schaltungstakt_ ){
		// Ablaufzeit = 30 Sekunden
		case 1:
				if( zeitLetzteSchaltungGlobal_ >= schaltzeit_ ){
					schalten = 1;
					zeitLetzteSchaltungGlobal_ = 0;
					mengeLetzteSchaltungGlobal_ = 0;
				}
				break;
		// Ablaufzeit = 1 Minute
		case 2:
				if( zeitLetzteSchaltungGlobal_ >= (schaltzeit_ * 2) ){
					schalten = 1;
					zeitLetzteSchaltungGlobal_ = 0;
					mengeLetzteSchaltungGlobal_ = 0;
				}
			    break;
	}


	// Schalte oder schalte nicht
	if( schalten != 0 ){
		schaltungstakt_++;
		if( schaltungstakt_ > 2 )
			schaltungstakt_ = 1;
		zeitLetzteSchaltungGlobal_ = 0;
		mengeLetzteSchaltungGlobal_ = 0;
	
		// Stelle die Ampeln entsprechend des aktuellen Taktes ein
		switch( schaltungstakt_ ){
			//	1. Nord gerade, Sued gerade, Nord rechts, Sued rechts bekommen gruen ... alle anderen rot
			case 1:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeRot();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeRot();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeRot();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeRot();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeRot();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeRot();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeRot();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeRot();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeRot();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeRot();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeRot();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeRot();

					break;
				}

				case 2:{
					if( spurenListe_[NORD-1][LINKS-1] != 0 )
						spurenListe_[NORD-1][LINKS-1]->setzeGruen();
					if( spurenListe_[NORD-1][MITTE-1] != 0 )
						spurenListe_[NORD-1][MITTE-1]->setzeGruen();
					if( spurenListe_[NORD-1][RECHTS-1] != 0 )
						spurenListe_[NORD-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[SUED-1][LINKS-1] != 0 )
						spurenListe_[SUED-1][LINKS-1]->setzeGruen();
					if( spurenListe_[SUED-1][MITTE-1] != 0 )
						spurenListe_[SUED-1][MITTE-1]->setzeGruen();
					if( spurenListe_[SUED-1][RECHTS-1] != 0 )
						spurenListe_[SUED-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[OST-1][LINKS-1] != 0 )
						spurenListe_[OST-1][LINKS-1]->setzeGruen();
					if( spurenListe_[OST-1][MITTE-1] != 0 )
						spurenListe_[OST-1][MITTE-1]->setzeGruen();
					if( spurenListe_[OST-1][RECHTS-1] != 0 )
						spurenListe_[OST-1][RECHTS-1]->setzeGruen();

					if( spurenListe_[WEST-1][LINKS-1] != 0 )
						spurenListe_[WEST-1][LINKS-1]->setzeGruen();
					if( spurenListe_[WEST-1][MITTE-1] != 0 )
						spurenListe_[WEST-1][MITTE-1]->setzeGruen();
					if( spurenListe_[WEST-1][RECHTS-1] != 0 )
						spurenListe_[WEST-1][RECHTS-1]->setzeGruen();

					break;
				}
		}
	}
}
