/*!
 * @file  Spur.cpp
 * @brief Implementierung der Klasse Spur.
 */

#include "Fehler.h"
#include "Konstanten.h"
#include "Spur.h"
class Fahrzeug;

/*!
 * @fn		Spur::Spur( )
 * @brief	Default-Konstruktor der Klasse Spur
 *
 * @author	Nico Schroeder
 */
Spur::Spur( ){
	clear();
}


/*!
 * @fn		Spur::Spur( long int id, double laenge, int maximalGeschwindigkeit)
 * @brief	ueberladener Konstruktor der Klasse Spur mit Einrichten der Spureigenschaften
 *
 * @author	Nico Schroeder
 *
 * @param	id Die ID der Spur zum besseren wiederfinden
 * @param	laenge Die Laenge der anzulegenden Spur.
 * @param	maximalGeschwindigkeit Die maximale Geschwindigkeit, mit der die Fahrzeuge auf der Spur fahren duerfen.
 */
Spur::Spur( long int id, double laenge, int maximalGeschwindigkeit){
	clear();
	id_ = id;
	laenge_ = laenge;
	maximalGeschwindigkeit_ = maximalGeschwindigkeit;	
}


/*!
 * @fn		int Spur::gibAnzahlFahrzeuge( )
 * @brief	gibt Anzahl der auf der Spur befindlichen Fahrzeuge zurueck, oder wenn Parameter angegeben wurden, die Anzahl der Fahrzeuge eines Types, die bisher ueber die Spur in eine bestimmte Richtung gefahren sind.
 *
 * @author	Nico Schroeder
 *
 * @return	Anzahl der Fahrzeuge auf der Spur
 */
int Spur::gibAnzahlFahrzeuge( ){
	return anzahlFahrzeuge_;
}

/*!
 * @fn		int Spur::gibAnzahlAbgefahrenerFahrzeuge( int typ )
 * @brief	gibt Anzahl der von der Spur abgefahrenen Fahrzeuge zurueck. Die Statistik wird dann wieder auf 0 gesetzt.
 *
 * @author	Nico Schroeder
 *
 * @param	typ Fahrzeugtyp, welcher interessiert (0 fuer alle Typen)
 * @return	Anzahl der Fahrzeuge
 * @throw	Fehler* Einen Fehler.
 */
int Spur::gibAnzahlAbgefahrenerFahrzeuge( int typ ){
	int anzahl;
	if( typ > 0 && typ < 5 ){
		anzahl = statistik_[typ-1];	
		statistik_[typ-1] = 0;
	}
	else if( typ == 0 ){
		anzahl = statistik_[0] + statistik_[1] + statistik_[2] + statistik_[3];
		statistik_[0] = 0;
		statistik_[1] = 0;
		statistik_[2] = 0;
		statistik_[3] = 0;
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDer Typ eines Fahrzeuges liegt ausserhalb der Grenzen.");
		throw(fehler);
	}

	return anzahl;
}

/*!
 * @fn		int Spur::gibEndzustand( )
 * @brief	gibt den Zustand am Ende der Spur zurueck
 *			Gibt zurueck, was am Ende der Spur fuer ein Zustand herrscht. Zum Beispiel Ampel rot (anhalten wegen
 *			Hinderniss) oder Ampel gruen (ungehinderte Fahrt).
 *
 * @author	Andreas Richter
 *
 * @return Gibt den Endzustand als int zurueck.
 */
int Spur::gibEndzustand( ){
	return endzustand_;
}


/*!
 * @fn		void Spur::setzeEndzustand(int zustand)
 * @brief	setzt den Zustand am Ender der Spur
 *			Setzt den Zustand am Ende der Spur, zum Beispiel Ampel rot (anhalten wegen Hinderniss) oder Ampel gruen
 *			(ungehinderte Fahrt).
 *
 * @author	Nico Schroeder
 * 
 * @param	zustand Flag das den Zustand angibt ( 1 = gruen | 2 = gelb | 3 = rot | 4 = sackgasse )
 * @throw	Fehler* Befindet sich der Parameter ausserhalb der erlaubten Grenzen wird ein Fehler geworfen.
 */
void Spur::setzeEndzustand(int zustand){
	if( zustand >= 0 && zustand < 4 ){
		endzustand_ = zustand;
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDas Endeflag fuer eine Spur einspricht nicht den Vorgaben.");
		throw(fehler);
	}
}



/*!
 * @fn		void Spur::setzeNaechstesVerbindungselement(Spur *spur, int richtung, double wahrscheinlichkeit)
 * @brief	knuepft neues Spurenelement am Ende an
 *
 * @author	Nico Schroeder
 *
 * @param   *spur Gibt die anzuhaengende Spur an
 * @param	richtung gibt die Richtung an ( 1 = links | 2 = gerade aus | 3 = rechts )
 * @param	wahrscheinlichkeit Setzt den Wahrscheinlichkeitswert fuer diese Spur (als p= 0 ... 1)
 * @throw	Fehler* Soll eine nicht existente Wahrscheinlichkeit oder eine Wahrscheinlichkeit fuer eine nicht existente Richtung gesetzt werden, wird ein Fehler geworfen.
 */
void Spur::setzeNaechstesVerbindungselement(Spur *spur, int richtung, double wahrscheinlichkeit){
	if( richtung > 0 && richtung < 4 && wahrscheinlichkeit <= 1 && wahrscheinlichkeit >= 0 ){
		spur_[richtung-1] = spur;
		wahrscheinlichkeit_[richtung-1] = wahrscheinlichkeit;
		spur->setzeAbbiegerichtung( richtung );
		setzeGruen();
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie Wahrscheinlichkeit fuer eine nicht existierende Richtung sollte gesetzt werden.");
		throw(fehler);
	}
}


/*!
 * @fn		double Spur::gibWahrscheinlichkeit(int richtung)
 * @brief	gibt wahrscheinlichkeit fuer das Befahren der naechsten Spur zurueck
 *			Gibt zurueck, wie grosz die Wahrscheinlichkeit ist, dass auf dieser Spur gefahren wird. Existiert die
 *			Spur nicht, wird die Wahrscheinlichkeit auf 0 gesetzt.
 *
 * @author	Nico Schroeder
 *
 * @param	richtung Flag fuer gewuenschte Richtung
 * @return	Liefert die Wahrscheinlichkeit zurueck, mit der auf der nszchsten Spur gefahren wird
 * @throw	Fehler* Existiert eine Richtung nicht, wird ein Fehler geworfen.
 */
double Spur::gibWahrscheinlichkeit(int richtung){
	if( richtung > 0 && richtung < 4 ){
		return wahrscheinlichkeit_[richtung-1];
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie Wahrscheinlichkeit fuer eine nicht existierende Richtung wurde abgefragt.");
		throw(fehler);
	}
	return 0;
}


/*!
 * @fn		void Spur::anmelden(Fahrzeug *fahrzeug)
 * @brief	Meldet ein Fahrzeug auf der Spur an --> dieses Fahrzeug faehrt nun auf der Spur.
 *
 * @author	Nico Schroeder
 *
 * @param	fahrzeug Dieses Fahrzeug anmelden
 * @throw	Fehler* Falls zu viele Fahrzeuge (>1000) auf der Spur fahren sollen, wird ein Fehler geworfen.
 */
void Spur::anmelden(Fahrzeug *fahrzeug){
	if( anzahlFahrzeuge_ < 1000){
		fahrzeugListe_[anzahlFahrzeuge_] = fahrzeug;
		anzahlFahrzeuge_++;
	}
	else{
		Fehler  *fehler = new Fehler("Auf einer Spur befinden sich mehr als 1000 Fahrzeuge.");
		throw( fehler );
	}
}


/*!
 * @fn		void Spur::abmelden(Fahrzeug *fahrzeug)
 * @brief	Meldet ein Fahrzeug von der Spur ab --> dieses Fahrzeug faehrt nun auf einer anderen Spur.
 *
 * @author	Nico Schroeder
 *
 * @param	fahrzeug Dieses Fahrzeug abmelden
 * @throw	Fehler* Wird das Fahrzeug nicht in der Liste gefunden, wird eine Fehlermeldung generiert.
 */
void Spur::abmelden(Fahrzeug *fahrzeug){
	int position(0), i(0);

	// Eigentlich ist das Fahrzeug welches sich abmeldet immer das erste --> Ueberpruefen
	if( fahrzeugListe_[0] != 0 && fahrzeugListe_[0]->gibId() == fahrzeug->gibId() ){
		position = 0;
	}
	else{
		// Sonst das Fahrzeug in der Liste suchen und die Position merken
		for( position = 0; position < 1000; position++){
			if( fahrzeugListe_[position] != 0 && fahrzeugListe_[position]->gibId() == fahrzeug->gibId() ){
				break;
			}
		}
		// Kommt das Programm hier an, dann gibt es das Fahrzeug nicht
		Fehler  *fehler = new Fehler("Aus einer Spur soll ein dort nicht befindliches Fahrzeug geloescht werden.");
		throw( fehler );
	}

	// Alle nach dem abzumeldenden Fahrzeug um eins nach vorn schieben
	for( i = position; i < 999; i++ ){
		fahrzeugListe_[i] = fahrzeugListe_[i+1];
	}
	fahrzeugListe_[999] = 0;

	anzahlFahrzeuge_--;

	// das Fahrzeug in die Statistiken aufnehmen
	statistik_[fahrzeug->gibTyp()-1]++;
}


/*!
 * @fn		double Spur::gibLaenge()
 * @brief	Liefert die Laenger der Spur.
 *
 * @author	Nico Schroeder
 *
 * @return	Die Laenge in m.
 */
double Spur::gibLaenge(){
	return laenge_;
}

/*!
 * @fn		Fahrzeug* Spur::gibVorgaenger(Fahrzeug *fahrzeug)
 * @brief	Liefert den Vorgaenger eines Fahrzeuges in der Spur zurueck.
 *
 * @author	Nico Schroeder
 *
 * @param	fahrzeug Das Fahrzeuges, fuer welches die Ueberpruefung erfolgen soll.
 * @return	Der Vorgaenger oder 0, wenn fahrzeug das erste und/oder einzigste Fahrzeug in der Spur ist.
 */
Fahrzeug* Spur::gibVorgaenger(Fahrzeug *fahrzeug){
	int position(0);

	for( position = 0; position < anzahlFahrzeuge_; position++){
		if( fahrzeugListe_[position]->gibId() == fahrzeug->gibId() ){
			if( position > 0 ){
				return fahrzeugListe_[position-1];
			}
		}
	}

	return 0;
}

/*!
 * @fn		Fahrzeug* Spur::gibLetztes()
 * @brief	Liefert das letzte Fahrzeug in der Spur.
 *
 * @author	Nico Schroeder
 *
 * @return	Die ID des Vorgaengers oder 0, wenn fahrzeugId das erste und/oder einzigste Fahrzeug ist.
 */
Fahrzeug* Spur::gibLetztes(){
	if( anzahlFahrzeuge_ > 1 ){
		return fahrzeugListe_[anzahlFahrzeuge_-1];
	}
	else{
		return 0;
	}
}

/*!
 * @fn		long int Spur::gibId()
 * @brief	Liefert die ID der Spur.
 *
 * @author	Nico Schroeder
 *
 * @return	Die ID der Spur.
 */
long int Spur::gibId(){
	return id_;
}


/*!
 * @fn		Spur *Spur::gibNaechsteSpur(int richtung)
 * @brief	Gibt die nachfolgende Spur in Richtung richtung
 *
 * @author	Nico Schroeder
 *
 * @param	richtung Die gewuenschte Spurrichtung.
 * @throw	Fehler* Ist der Parameter ausserhalb der zulaessigen Grenzen, wird ein Fehler geworfen.
 */
Spur *Spur::gibNaechsteSpur(int richtung){
	if( richtung > 0 && richtung < 4 && spur_[richtung-1] != 0 ){
		return spur_[richtung-1];
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie gewuenschte Spur existiert nicht.");
		throw(fehler);
	}
}


/*!
 * @fn		double Spur::gibMaximaleGeschwindigkeit()
 * @brief	Liefert die lokale Hoechstgeschwindigkeit auf der Spur
 *
 * @author	Nico Schroeder
 *
 * @return	Die maximale Geschwindigkeit.
 */
double Spur::gibMaximaleGeschwindigkeit(){
	return maximalGeschwindigkeit_;
}


/*!
 * @fn		void Spur::clear()
 * @brief	Setzt die Spur auf Standardwerte
 *
 * @author	Nico Schroeder
 */
void Spur::clear(){
	int i(0);
	for( i=0; i<1000; i++){
		fahrzeugListe_[i] = 0;
	}
	for( i = 0; i<4; i++ ){
		statistik_[i] = 0;
	}

	wahrscheinlichkeit_[0] = 0;
	wahrscheinlichkeit_[1] = 0;
	wahrscheinlichkeit_[2] = 0;
	spur_[0] = 0;
	spur_[1] = 0;
	spur_[2] = 0;
	nachfolger_[0] = 0;
	nachfolger_[1] = 0;
	nachfolger_[2] = 0;
	maximalGeschwindigkeit_ = 50;
	laenge_ = 300;
	endzustand_ = STOP;		// immer erst einmal auf eine Sackgasse setzen
	anzahlFahrzeuge_ = 0;
	abbiegerichtung_ = NICHTS;
	richtung_ = MITTE;
}


/*!
 * @fn		void Spur::setzeGruen()
 * @brief	Setzt die Ampel am Ende der Spur auf Gruen
 *
 * @author	Nico Schroeder
 */
void Spur::setzeGruen(){
	setzeEndzustand(GRUEN);
}


/*!
 * @fn		void Spur::setzeGelb()
 * @brief	Setzt die Ampel am Ende der Spur auf Gelb
 *
 * @author	Nico Schroeder
 */
void Spur::setzeGelb(){
	setzeEndzustand(GELB);
}


/*!
 * @fn		void Spur::setzeRot()
 * @brief	Setzt die Ampel am Ende der Spur auf Rot
 *
 * @author	Nico Schroeder
 */
void Spur::setzeRot(){
	setzeEndzustand(ROT);
}


/*!
 * @fn		void Spur::setzeSackgasse()
 * @brief	Am Ende der Spur schliesst keine weitere Spur an.
 *
 * @author	Nico Schroeder
 */
void Spur::setzeSackgasse(){
	setzeEndzustand(STOP);
}


/*!
 * @fn		bool Spur::istGruen()
 * @brief	Gibt an, ob die Ampel am Ende der Spur auf Gruen steht.
 *
 * @author	Nico Schroeder
 *
 * @return	Wahr, wenn Ampel Gruen ist, sonst Falsch.
 */
bool Spur::istGruen(){
	if( gibEndzustand() == GRUEN )
		return 1;
	else
		return 0;
}


/*!
 * @fn		bool Spur::istGelb()
 * @brief	Gibt an, ob die Ampel am Ende der Spur auf Gelb steht.
 *
 * @author	Nico Schroeder
 *
 * @return	Wahr, wenn Ampel Gelb ist, sonst Falsch.
 */
bool Spur::istGelb(){
	if( gibEndzustand() == GELB )
		return 1;
	else
		return 0;
}


/*!
 * @fn		bool Spur::istRot()
 * @brief	Gibt an, ob die Ampel am Ende der Spur auf Rot steht.
 *
 * @author	Nico Schroeder
 *
 * @return	Wahr, wenn Ampel Rot ist, sonst Falsch.
 */
bool Spur::istRot(){
	if( gibEndzustand() == ROT )
		return 1;
	else
		return 0;
}


/*!
 * @fn		bool Spur::istSackgasse()
 * @brief	Gibt an, ob die Spur eine Sackgasse ist.
 *
 * @author	Nico Schroeder
 *
 * @return	Wahr, wenn die Spur eine Sackgasse ist, sonst Falsch.
 */
bool Spur::istSackgasse(){
	if( gibEndzustand() == STOP )
		return 1;
	else
		return 0;
}

/*!
 * @fn		Koordinaten Spur::gibKoordinaten()
 * @brief	Gibt die Koordinaten der Spur zurueck
 *
 * @author	Nico Schroeder
 *
 * @return	Die Koordinaten der Spur
 */
Koordinaten Spur::gibKoordinaten(){
	return koordinaten_;
}

/*!
 * @fn		void Spur::setzeKoordinaten( Koordinaten koordinaten )
 * @brief	Setzt die Koordinaten der Spur.
 *
 * @author	Nico Schroeder
 *
 * @param	koordinaten Die neuen Koordinaten
 */
void Spur::setzeKoordinaten( Koordinaten koordinaten ){
	koordinaten_ = koordinaten;
}

/*!
 * @fn		int Spur::gibAusrichtung()
 * @brief	Gibt die Ausrichtung einer Spur zurueck.
 *
 * @author	Nico Schroeder
 *
 * @return	Die Ausrichtung als HOR oder VER
 */
int Spur::gibAusrichtung(){
	return ausrichtung_;
}

/*!
 * @fn		void Spur::setzeAusrichtung( int ausrichtung )
 * @brief	Setzt die Ausrichtung der Spur.
 *
 * @author	Nico Schroeder
 *
 * @param	ausrichtung Die neue Ausrichtung
 */
void Spur::setzeAusrichtung( int ausrichtung ){
	ausrichtung_ = ausrichtung;
}

/*!
 * @fn		int Spur::gibRichtung()
 * @brief	Gibt die Richtung einer Spur zurueck.
 *
 * @author	Nico Schroeder
 *
 * @return	Die Ausrichtung als RTOL, LTOR, DOWN oder UP
 */
int Spur::gibRichtung(){
	return richtung_;
}

/*!
 * @fn		void Spur::setzeRichtung( int richtung )
 * @brief	Setzt die Richtung der Spur.
 *
 * @author	Nico Schroeder
 *
 * @param	richtung Die neue Richtung
 */
void Spur::setzeRichtung( int richtung ){
	richtung_ = richtung;
}

/*!
 * @fn		int Spur::gibAbbiegerichtung()
 * @brief	Gibt die Abbiegerichtung der Spur.
 *
 * @author	Nico Schroeder
 *
 * @return	Die Abbiegerichtung der Spur
 */
int Spur::gibAbbiegerichtung(){
	return abbiegerichtung_;
}

/*!
 * @fn		void Spur::setzeAbbiegerichtung( int abbiegerichtung )
 * @brief	Setzt die Abbiegerichtung der Spur.
 *
 * @author	Nico Schroeder
 *
 * @param	abbiegerichtung Die neue Abbiegerichtung
 * @throw	Fehler* Wenn die Abbiegerichtung ausserhalb zulaessiger Paramet ist, wird ein Fehler geworfen.
 */
void Spur::setzeAbbiegerichtung( int abbiegerichtung ){
	if( abbiegerichtung > 0 && abbiegerichtung < 4){
		abbiegerichtung_ = abbiegerichtung;
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nEs soll eine Abbiegerichtung gesetzt werden, die nicht moeglich ist.");
		throw(fehler);
	}
	
}

/*!
 * @fn		long int Spur::gibNachfolgerId(int richtung)
 * @brief	Setzt die ID einer nachfolgender Spur
 *
 * @author	Nico Schroeder
 *
 * @return	Die gewuenschte ID
 * @throw	Fehler* Soll eine ID fuer eine nicht existente Richtung gesetzt werden, wird ein Fehler geworfen.
 */
long int Spur::gibNachfolgerId(int richtung){
	if( richtung > 0 && richtung < 4 && nachfolger_[richtung-1] != 0){
		return nachfolger_[richtung-1];
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie Richtung eines Nachfolgers ist ausserhalb der zulaessigen Parameter.");
		throw(fehler);
	}
}

/*!
 * @fn		void Spur::setzeNaechstesVerbindungselement(int richtung, long int id, double wahrscheinlichkeit)
 * @brief	Setzt die ID einer nachfolgender Spur
 *
 * @author	Nico Schroeder
 *
 * @param	richtung gibt die Richtung an ( 1 = links | 2 = gerade aus | 3 = rechts )
 * @param	id Die ID der nachfolgenden Spur
 * @param	wahrscheinlichkeit Die Wahrscheinlichkeit mit der diese Spur (id) als naechstes befahren wird
 * @throw	Fehler* Soll eine ID fuer eine nicht existente Richtung gesetzt werden, wird ein Fehler geworfen.
 */
void Spur::setzeNaechstesVerbindungselement(int richtung, long int id, double wahrscheinlichkeit){
	if( richtung > 0 && richtung < 4 && wahrscheinlichkeit <= 1 && wahrscheinlichkeit >= 0 ){
		nachfolger_[richtung-1] = id;
		wahrscheinlichkeit_[richtung-1] = wahrscheinlichkeit;
	}
	else{
		Fehler *fehler = new Fehler("Parameter ausserhalb zulaessiger Grenzen.\nDie Richtung eines Nachfolgers ist ausserhalb der zulaessigen Parameter.");
		throw(fehler);
	}
}

/*!
 * @fn		ostream &operator<< ( ostream &os, Spur *spur )
 * @brief	Ueberladener Operator "<<" z.B. zur Ausgabe einer Spur ueber cout
 *
 * @author	Nico Schroeder
 */
ostream &operator<< ( ostream &os, Spur *spur ){
	os << "----------Spur----------" << endl;
	os << "ID:                   " << spur->id_ << endl;
	os << "Endzustand:           " << spur->endzustand_ << "( 1 = gruen | 2 = gelb | 3 = rot | 0 = sackgasse )" << endl;
	os << "Laenge:               " << spur->laenge_ << " m" << endl;
	os << "max. Geschwindigkeit: " << spur->maximalGeschwindigkeit_ << " km/h" << endl;
	os << "p fuer links:         " << spur->wahrscheinlichkeit_[0] << endl;
	os << "p fuer gerade:        " << spur->wahrscheinlichkeit_[1] << endl;
	os << "p fuer rechts:        " << spur->wahrscheinlichkeit_[2] << endl;
	os << "Anzahl Fahrzeuge:     " << spur->anzahlFahrzeuge_ << endl;
	return os;
}