/*!
 * @file  SzenarioParser.cpp
 * @brief Implementierung der Klassen SzenarioParser.
 */

#include "SzenarioParser.h"
#include "StructureParser.h"

#include "../Konstanten.h"
#include "../Liste.h"
#include "../Spur.h"
#include "../Fehler.h"

#include <qfile.h>
#include <qxml.h>

//------------------------------------------------------------------------------------------------------------------
/*!
 * @fn		SzenarioParser::SzenarioParser()
 * @brief	Standardkonstruktor der Klasse SzenarioParser.
 *
 * @author	Andreas Richter
 */
SzenarioParser::SzenarioParser()
{
	// please do not use
}
//------------------------------------------------------------------------------------------------------------------



//------------------------------------------------------------------------------------------------------------------
/*!
 * @fn		SzenarioParser::SzenarioParser( char *file, Spur ***spurenArray, Liste *generatorListe, Liste *kreuzungsListe, Liste *fahrzeugeListe )
 * @brief	Ueberladener Konstruktor der Klasse SzenarioParser, der die Pointer zu den Arrays speichert.
 *
 * @author	Andreas Richter
 *
 * @param   file Dateipfad zur XML-Datei
 * @param   spurenArray Zeiger auf das erste Element des SpurenArrays.
 * @param   generatorListe Zeiger auf die Liste der Fahrzeuggeneratoren.
 * @param	kreuzungsListe Zeiger auf die Liste der Kreuzungen.
 * @param	fahrzeugeListe Zeiger auf die Liste der Fahrzeuge.
 */
SzenarioParser::SzenarioParser( char *file, Spur ***spurenArray, Liste *generatorListe, Liste *kreuzungsListe, Liste *fahrzeugeListe )
{
	          file_ = file;
	   spurenArrayTmp_ = spurenArray;
	generatorListe_ = generatorListe;		
	kreuzungsListe_ = kreuzungsListe;
	fahrzeugeListe_ = fahrzeugeListe;
}
//------------------------------------------------------------------------------------------------------------------



//------------------------------------------------------------------------------------------------------------------
/*!
 * @fn		int SzenarioParser::parseXml()
 * @brief	Parst das uebergebenen XML-Dokument.
 *
 * @author	Andreas Richter
 *
 * @return  Liefert Anzahl der Spuren in der XML-Datei zurueck.
 *
 * @throw	Wirft einen Fehler, wenn keine XML-Datei angegeben wurde oder illegale Speicherzugriffe auftreten
 */
int SzenarioParser::parseXml()
{
	int anzahlSpuren;
	long int maxID, nextID;
	double  wahrschLinks = 0, wahrschMitte = 0, wahrschRechts = 0;

	// Anzahl der Spuren im XML-File feststellen
	anzahlSpuren = countXml();

	// groesste SpurenID fesstellen
	maxID = getXml();


	// Array aller Spuren in Lesereihenfolge
	*spurenArrayTmp_ = new Spur *[anzahlSpuren];
	spurenArray_ = *spurenArrayTmp_;

	

	// Hilfsarray der die Spuren im Spurenarray auffindbar macht
	long int *spurenPosition_;
	spurenPosition_ = new long int[maxID+1];
	
	for( int i=0; i<=maxID; i++ )
		spurenPosition_[i] = -1;
	
	
	QFile xmlFile( file_ );

	if ( !xmlFile.exists() )
	{
		Fehler *fehler = new Fehler("XML-Datei konnte nicht gefunden werden.");
		throw( fehler );
	}


	// XML-Dokument parsen und Listen befuellen
	StructureParser  handler( spurenArray_, spurenPosition_, generatorListe_, kreuzungsListe_, fahrzeugeListe_ );

	QXmlInputSource source( &xmlFile );
	QXmlSimpleReader speichern;
	
	speichern.setContentHandler( &handler );
	try
	{
		speichern.parse( source );
	}
	catch(Fehler *fehler)
	{
		xmlFile.close();
		throw( fehler);
	}
	
	xmlFile.close();
	
	// Spuren im Speicher organisieren
	for( int i = 0; i < anzahlSpuren; i++ )
	{
		// gibt es einen Nachfolger, dann:
		// ID des Nachfolgers besorgen und Spur als Nachfolger einhaengen
		wahrschLinks = spurenArray_[ i ]->gibWahrscheinlichkeit( LINKS );
		if( wahrschLinks != 0 )
		{
			try
			{
				nextID = spurenArray_[ i ]->gibNachfolgerId( LINKS );
			}
			catch(Fehler *f)
			{
				Fehler *fehler = new Fehler("Eine als Nachfolger deklarierte Spur ist in der XML-Datei nicht selbst definiert worden.");
				throw( fehler );
			}
			
			spurenArray_[ i ]->setzeNaechstesVerbindungselement( spurenArray_[ spurenPosition_[nextID] ] , LINKS, wahrschLinks );
		}
		
		wahrschMitte = spurenArray_[ i ]->gibWahrscheinlichkeit( MITTE );
		if( wahrschMitte != 0 )
		{
			try
			{
				nextID = spurenArray_[ i ]->gibNachfolgerId( MITTE );
			}
			catch(Fehler *f)
			{
				Fehler *fehler = new Fehler("Eine als Nachfolger deklarierte Spur ist in der XML-Datei nicht selbst definiert worden.");
				throw( fehler );
			}

			spurenArray_[ i ]->setzeNaechstesVerbindungselement( spurenArray_[ spurenPosition_[nextID] ] , MITTE, wahrschMitte );
		}
		
		wahrschRechts = spurenArray_[ i ]->gibWahrscheinlichkeit( RECHTS );
		if( wahrschRechts != 0 )
		{
			try
			{
				nextID = spurenArray_[ i ]->gibNachfolgerId( RECHTS );
			}
			catch(Fehler *f)
			{
				Fehler *fehler = new Fehler("Eine als Nachfolger deklarierte Spur ist in der XML-Datei nicht selbst definiert worden.");
				throw( fehler );
			}
			spurenArray_[ i ]->setzeNaechstesVerbindungselement( spurenArray_[ spurenPosition_[nextID] ] , RECHTS, wahrschRechts );
		}
	}
	
	return anzahlSpuren;
}
//------------------------------------------------------------------------------------------------------------------



//------------------------------------------------------------------------------------------------------------------
/*!
 * @fn		int SzenarioParser::countXml()
 * @brief	Zaehlt im uebergebenem XML-Dokument bestimmte Elemente. (Hier allerdings nur die Spuren.)
 *
 * @author	Andreas Richter
 *
 * @return  Liefert Anzahl der Spuren in der XML-Datei zurueck.
 */
int SzenarioParser::countXml()
{
	QFile xmlFile( file_ );
	
	StructureCounter counter;
	
	QXmlInputSource source( &xmlFile );
	QXmlSimpleReader zaehlen;	
	
	zaehlen.setContentHandler( &counter );
	zaehlen.parse( source );
	
	return counter.gibAnzahlSpuren();
}
//------------------------------------------------------------------------------------------------------------------



//------------------------------------------------------------------------------------------------------------------
/*!
 * @fn		SzenarioParser::getXml()
 * @brief	Ermittelt im uebergebenem XML-Dokument groesste SpurenID.
 *
 * @author	Andreas Richter
 *
 * @return  Gibt die groesste benutzte ID einer Spur in der XML-Datei zurueck.
 */
long int SzenarioParser::getXml()
{
	QFile xmlFile( file_ );
	
	StructureGetter getter;
	
	QXmlInputSource source( &xmlFile );
	QXmlSimpleReader idLeser;	
	
	idLeser.setContentHandler( &getter );
	idLeser.parse( source );
	
	return getter.gibMaxId();
}
//------------------------------------------------------------------------------------------------------------------